@@ -3,6 +3,7 @@ extern crate difference;
 extern crate clap;
 
 use clap::{App, Arg};
+use std::error;
 use std::fs::File;
 use std::io;
 use std::io::{BufRead, BufReader};
@@ -16,17 +17,59 @@ use difference::Difference::{Add, Rem, Same};
 extern crate prettytable;
 use prettytable::Table;
 
-fn get_line_from_file(filename: &Path) -> String {
-    if !filename.exists() || !filename.is_file() {
-        println!("Cannot find file1: {:?}", filename);
-        return "".to_string();
+type Result<T> = std::result::Result<T, Box<dyn error::Error>>;
+
+struct LineData {
+    name: String,
+    line: String,
+    preprocessed: String,
+}
+
+impl LineData {
+    fn new(name: &str, line: &str) -> LineData {
+        LineData {
+            name: name.to_string(),
+            line: line.to_string(),
+            preprocessed: "".to_string(),
+        }
     }
-    let file = File::open(filename).unwrap();
+
+    fn length(&self) -> usize {
+        self.line.chars().count()
+    }
+
+    fn number_chunks(&self) -> usize {
+        self.preprocessed.matches('\n').count() + 1
+    }
+
+    fn preprocess_chunks(&mut self, separator: &[char], sort: bool) {
+        let mut chunks: Vec<&str> = self.line.split(|c| separator.contains(&c)).collect();
+        if sort {
+            chunks.sort();
+        }
+        self.preprocessed = chunks.join("\n");
+    }
+}
+
+fn verify_existing_file(path: &Path) -> Result<()> {
+    if !path.exists() {
+        Err(format!("Cannot find file1: {}", path.display()).into())
+    } else if !path.is_file() {
+        Err(format!("Is not a file: {}", path.display()).into())
+    } else {
+        Ok(())
+    }
+}
+
+fn get_line_from_file(path: &Path) -> Result<LineData> {
+    verify_existing_file(path)?;
+
+    let file = File::open(path)?;
     let reader = BufReader::new(file);
 
     let mut s = "".to_owned();
     for (index, line) in reader.lines().enumerate() {
-        let line = line.unwrap();
+        let line = line?;
 
         if index == 0 {
             s = line.to_owned();
@@ -35,11 +78,22 @@ fn get_line_from_file(filename: &Path) -> String {
             break;
         }
     }
-    s.to_string()
+    let file_name = if let Some(file_name2) = path.file_name() {
+        if let Ok(file_name3) = file_name2.to_os_string().into_string() {
+            file_name3
+        } else {
+            "".into()
+        }
+    } else {
+        "".into()
+    };
+    Ok(LineData::new(&file_name, &s))
 }
 
-fn get_lines_from_file(filename: &Path) -> (String, String) {
-    let file = File::open(filename).unwrap();
+fn get_lines_from_file(path: &Path) -> Result<(LineData, LineData)> {
+    verify_existing_file(path)?;
+
+    let file = File::open(path).unwrap();
     let reader = BufReader::new(file);
 
     let mut s1 = "".to_owned();
@@ -56,45 +110,39 @@ fn get_lines_from_file(filename: &Path) -> (String, String) {
             break;
         }
     }
-    (s1.to_string(), s2.to_string())
+    Ok((LineData::new("Line 1", &s1), LineData::new("Line 2", &s2)))
 }
 
-fn get_line_from_cmd(line_number: i32) -> String {
+fn get_line_from_cmd(line_number: i32) -> LineData {
     println!("Please provide line #{}: ", line_number);
     let mut buffer = String::new();
     io::stdin().read_line(&mut buffer).expect("");
-    buffer.trim().to_string()
+    LineData::new(&format!("Line {}", line_number), &buffer.trim().to_string())
 }
 
-fn get_line(line_number: i32, filepath: Option<&str>) -> String {
+fn get_line(line_number: i32, filepath: Option<&str>) -> Result<LineData> {
     match filepath {
         Some(filepath) => get_line_from_file(Path::new(filepath)),
-        None => get_line_from_cmd(line_number),
+        None => Ok(get_line_from_cmd(line_number)),
     }
 }
 
-fn print_results(diffs: Vec<Difference>) {
+fn print_results(l1: &LineData, l2: &LineData, diffs: Vec<Difference>) {
     let mut table = Table::new();
-    table.add_row(row!["Line 1", "Same", "Line 2"]);
-    for d in diffs {
+    table.add_row(row![bFgc => l1.name, "Same", l2.name]);
+    for d in diffs.iter() {
         match d {
             Same(line) => table.add_row(row!["", line, ""]),
             Add(line) => table.add_row(row!["", "", line]),
             Rem(line) => table.add_row(row![line, "", ""]),
         };
     }
+    table.add_row(row![bFgc => l1.length(), "Characters", l2.length()]);
+    table.add_row(row![bFgc => l1.number_chunks(), "Chunks", l2.number_chunks()]);
     table.printstd();
 }
 
-fn preprocess_chunks(s: &str, separator: &[char], sort: bool) -> String {
-    let mut chunks: Vec<&str> = s.split(|c| separator.contains(&c)).collect();
-    if sort {
-        chunks.sort();
-    }
-    chunks.join("\n")
-}
-
-fn main() {
+fn main() -> Result<()> {
     let matches = App::new("Line diff")
         .version(crate_version!())
         .author(crate_authors!())
@@ -104,6 +152,7 @@ fn main() {
         \tspecifying the two lines separately as a file path (indexed argument 1 and 2), on the command line (--line1 and --line2) or using command line input.")
         .arg(
             Arg::with_name("file")
+                .long("file")    
                 .help("A single file containing two lines. Remaining lines will be ignored.")
                 .short("f")
                 .takes_value(true),
@@ -120,12 +169,14 @@ fn main() {
         )
         .arg(
             Arg::with_name("line1")
+                .short("x")
                 .long("line1")
                 .help("First line as string")
                 .takes_value(true),
         )
         .arg(
             Arg::with_name("line2")
+                .short("y")
                 .long("line2")
                 .help("Second line as string")
                 .takes_value(true),
@@ -144,23 +195,20 @@ fn main() {
         )
         .get_matches();
 
-    let (s1, s2) = if let Some(filepath) = matches.value_of("file") {
+    let (mut s1, mut s2) = if let Some(filepath) = matches.value_of("file") {
         let path_file = Path::new(filepath);
-        if !path_file.exists() || !path_file.is_file() {
-            println!("Cannot find file1: {}", filepath);
-            return;
-        }
-        get_lines_from_file(path_file)
+        verify_existing_file(path_file)?;
+        get_lines_from_file(path_file)?
     } else {
         let l1 = if let Some(l1) = matches.value_of("line1") {
-            l1.to_owned()
+            LineData::new("Line 1", &l1)
         } else {
-            get_line(1, matches.value_of("file1"))
+            get_line(1, matches.value_of("file1"))?
         };
         let l2 = if let Some(l2) = matches.value_of("line2") {
-            l2.to_owned()
+            LineData::new("Line 2", &l2)
         } else {
-            get_line(2, matches.value_of("file2"))
+            get_line(2, matches.value_of("file2"))?
         };
         (l1, l2)
     };
@@ -180,14 +228,15 @@ fn main() {
     } else {
         vec![' ']
     };
-    println!("Line 1: \n{}", s1);
-    println!("Line 2: \n{}", s2);
+    println!("{}: \n{}", s1.name, s1.line);
+    println!("{}: \n{}", s2.name, s2.line);
 
-    let s1 = preprocess_chunks(&s1, &separator_chars, sort);
-    let s2 = preprocess_chunks(&s2, &separator_chars, sort);
+    s1.preprocess_chunks(&separator_chars, sort);
+    s2.preprocess_chunks(&separator_chars, sort);
 
-    let changeset = Changeset::new(&s1, &s2, "\n");
-    print_results(changeset.diffs);
+    let changeset = Changeset::new(&s1.preprocessed, &s2.preprocessed, "\n");
+    print_results(&s1, &s2, changeset.diffs);
+    Ok(())
 }
 
 #[cfg(test)]
@@ -195,40 +244,56 @@ mod tests {
     use super::*;
     #[test]
     fn preprocess_no_sorting() {
-        let output = preprocess_chunks("hello world", &vec![' '], false);
-        assert_eq!("hello\nworld", output);
-        let output = preprocess_chunks("hello world", &vec![';'], false);
-        assert_eq!("hello world", output);
-        let output = preprocess_chunks("hello world", &vec!['o'], false);
-        assert_eq!("hell\n w\nrld", output);
+        let mut data = LineData::new("Line 1", "hello world");
+        data.preprocess_chunks(&vec![' '], false);
+        assert_eq!("hello\nworld", data.preprocessed);
+
+        let mut data = LineData::new("Line 1", "hello world");
+        data.preprocess_chunks(&vec![';'], false);
+        assert_eq!("hello world", data.preprocessed);
+
+        let mut data = LineData::new("Line 1", "hello world");
+        data.preprocess_chunks(&vec!['o'], false);
+        assert_eq!("hell\n w\nrld", data.preprocessed);
     }
 
     #[test]
     fn preprocess_sorting() {
-        let output = preprocess_chunks("a b c", &vec![' '], true);
-        assert_eq!("a\nb\nc", output);
-        let output = preprocess_chunks("c b a", &vec![' '], true);
-        assert_eq!("a\nb\nc", output);
+        let mut data = LineData::new("Line 1", "a b c");
+        data.preprocess_chunks(&vec![' '], true);
+        assert_eq!("a\nb\nc", data.preprocessed);
+
+        let mut data = LineData::new("Line 1", "c b a");
+        data.preprocess_chunks(&vec![' '], true);
+        assert_eq!("a\nb\nc", data.preprocessed);
     }
 
     #[test]
     fn preprocess_multiple_separators() {
-        let output = preprocess_chunks("a b;c", &vec![' '], true);
-        assert_eq!("a\nb;c", output);
-        let output = preprocess_chunks("c b a", &vec![' ', ';'], true);
-        assert_eq!("a\nb\nc", output);
+        let mut data = LineData::new("Line 1", "a b;c");
+        data.preprocess_chunks(&vec![' '], true);
+        assert_eq!("a\nb;c", data.preprocessed);
+
+        let mut data = LineData::new("Line 1", "c b a");
+        data.preprocess_chunks(&vec![' ', ';'], true);
+        assert_eq!("a\nb\nc", data.preprocessed);
     }
 
     #[test]
-    fn read_one_line() {
-        let l1 = get_line_from_file(Path::new("test.txt"));
-        assert_eq!("Hello world 1 3 .", l1);
+    fn read_one_line() -> Result<()> {
+        let l1 = get_line_from_file(Path::new("test.txt"))?;
+        assert_eq!("test.txt", l1.name);
+        assert_eq!("Hello world 1 3 .", l1.line);
+        Ok(())
     }
 
     #[test]
-    fn read_two_lines() {
-        let (l1, l2) = get_lines_from_file(Path::new("test.txt"));
-        assert_eq!("Hello world 1 3 .", l1);
-        assert_eq!("as the %+3^ night", l2);
+    fn read_two_lines() -> Result<()> {
+        let (l1, l2) = get_lines_from_file(Path::new("test.txt"))?;
+        assert_eq!("Line 1", l1.name);
+        assert_eq!("Line 2", l2.name);
+        assert_eq!("Hello world 1 3 .", l1.line);
+        assert_eq!("as the %+3^ night", l2.line);
+        Ok(())
     }
 }
